/*
	Tester.java is the source to the test program. The compiled version of this file
	is included in JConfig.zip.

	Copyright  1997 Samizdat Productions. All Rights Reserved.
*/

package COM.tolstoy.jconfig;

import java.awt.*;
import java.awt.image.*;
import java.io.File;
import java.io.PrintStream;

/**
This class is used to test this package on Mac.
*/

public class Tester {
	private static final String copyrightString = "JConfig Copyright (c) 1997 Samizdat Productions. All Rights Reserved.";
	public static final int		kMySignature = JUtils.asciiToInt( "fred" );

	public static void main( String args[] ) {
		FinderInfo			finfo;
		FileExtension		txtExtension;
		AppFile				simpleAppFile = null;
		AppProcess			simpleAppProcess = null;
		File				curDir;
		
		try {
			curDir = new File( System.getProperty( "user.dir" ) );

			FileRegistry.initialize( curDir, kMySignature );
			if ( !FileRegistry.isInited() ) {
				System.out.println( "Please check your configuration." );
				return;
			}

			finfo = new FinderInfo( JUtils.asciiToInt( "ttxt" ), JUtils.asciiToInt( "TEXT" ) );
			txtExtension = new FileExtension( ".txt" );

			simpleAppFile = tryFindAppByFinderInfo( finfo );

			tryFindAppByExtension( txtExtension );
			
			if ( simpleAppFile != null ) {
				tryGetFileTypes( simpleAppFile );
				simpleAppProcess = tryOpenApp( simpleAppFile );
			}

			tryIteratingOnConfigEntries();

			tryFindFinderInfo( txtExtension );

			tryFindExtensions( finfo );

			tryFindAppsByName( "ed" );

			if ( simpleAppProcess != null ) {
				tryOpenFile( simpleAppProcess );
				tryMoveAppToFront( simpleAppProcess );
			}

			if ( simpleAppFile != null )
				tryDisplayApplicationIcon( simpleAppFile, IconBundle.kLargeIcon );
			
			tryLaunchingURL( "http://www.tolstoy.com" );
		}
		catch ( Exception e ) {
			System.out.println( "problems: " + e );
			e.printStackTrace( System.out );
		}

		System.out.println( "" );
		System.out.println( "*** Ciao." );
	}

/*
Try to launch the user's browser. 'url' must be a full url, i.e., complete with 'http', or 'mailto',
etc.
*/

	private static int tryLaunchingURL( String url ) {
		int			theErr;

		System.out.println( "" );
		System.out.println( "*** Launching a URL: " + url );

		theErr = FileRegistry.launchURL( url );
		
		if ( theErr != 0 )
			System.out.println( "can't launch browser, theErr=" + theErr );
		
		return theErr;
	}

/*
Using the FileRegistry, try to find an application corresponding to the given FinderInfo.
*/

	private static AppFile tryFindAppByFinderInfo( FinderInfo finfo ) {
		AppFile				theApps[];

		System.out.println( "" );
		System.out.println( "*** Calling FileRegistry.getApps for " + finfo );

		theApps = FileRegistry.getApps( finfo, 1 );
		if ( theApps == null || theApps[ 0 ] == null ) {
			System.out.println( "can't find apps using " + finfo );
			return null;
		}

		theApps[ 0 ].dumpInfo( System.out, "  " );
		
		return theApps[ 0 ];
	}

/*
Using the FileRegistry, try to find an application corresponding to the given FileExtension.
*/

	private static AppFile tryFindAppByExtension( FileExtension ext ) {
		AppFile				theApps[];

		System.out.println( "" );
		System.out.println( "*** Calling FileRegistry.getApps for " + ext );

		theApps = FileRegistry.getApps( ext, 1 );
		if ( theApps == null || theApps[ 0 ] == null ) {
			System.out.println( "can't find apps using " + ext );
			return null;
		}

		theApps[ 0 ].dumpInfo( System.out, "  " );
		
		return theApps[ 0 ];
	}

/*
Try to print out the file types associated with the given application.
*/

	private static void tryGetFileTypes( AppFile appFile ) {
		FileType			fileTypes[];
		int					i;
	 	
		System.out.println( "" );
		System.out.println( "*** Getting the FileTypes of an application..." );

		fileTypes = appFile.getFileTypes( 10 );
		if ( fileTypes == null ) {
			System.out.println( "application has no file types" );
			return;
		}

		System.out.println( "application has " + fileTypes.length + " file types:" );

		for ( i = 0; i < fileTypes.length; i++ )
			System.out.println( "  " + fileTypes[ i ] );
	}

/*
Try to create an instance of the given application. Returns the resulting AppProcess, or null
if an error occurs.
*/

	private static AppProcess tryOpenApp( AppFile appFile ) {
		AppCommand			theCommand;
		AppProcess			theProcess;

		System.out.println( "" );
		System.out.println( "*** Calling performCommand to open an application..." );

		theCommand = appFile.getCommand( AppCommand.kAppCommandOpenApp );
		if ( theCommand == null ) {
			System.out.println( "can't get kAppCommandOpenApp from process" );
			return null;
		}

		System.out.println( "theCommand = " + theCommand );

		theProcess = appFile.performCommand( theCommand, AppFile.kDontSwitchLayer );
		if ( theProcess == null )
			System.out.println( "couldn't create process" );
		else {
			theProcess.dumpInfo( System.out, "" );
			System.out.println( "The AppFile now should have one spawn:" );
			appFile.dumpInfo( System.out, "  " );
		}

		return theProcess;
	}

/*
Display every 20th entry in the file mappings database.
*/

	private static void tryIteratingOnConfigEntries() {
		System.out.println( "" );
		System.out.println( "*** Iterating on ConfigEntry's..." );
		
		FileRegistry.iterate( new BasicCEVisitor( System.out ) );
	}

/*
Try to find FinderInfo objects corresponding to the given FileExtension.
*/

	private static void tryFindFinderInfo( FileExtension ext ) {
		FinderInfo		infoArray[];
		int				i, numWritten;

		System.out.println( "" );
		System.out.println( "*** Finding FinderInfo's associated with extension: " + ext );

		infoArray = FileRegistry.findFinderInfo( ext, 10 );
		if ( infoArray == null ) {
			System.out.println( "can't find finder info for " + ext );
			return;
		}

		for ( i = 0, numWritten = 0; i < infoArray.length; i++ ) {
			if ( infoArray[ i ] != null ) {
				System.out.println( "at " + i + " is " + infoArray[ i ] );
				++numWritten;
			}
		}
		
		if ( numWritten == 0 )
			System.out.println( "can't find finder info for " + ext );
	}

/*
Try to find FileExtension objects corresponding to the given FinderInfo.
*/

	private static void tryFindExtensions( FinderInfo finfo ) {
		FileExtension		extArray[];
		int					i, numWritten;

		System.out.println( "" );
		System.out.println( "*** Finding extensions associated with FinderInfo: " + finfo );

		extArray = FileRegistry.findExtensions( finfo, 10 );
		if ( extArray == null ) {
			System.out.println( "can't find extensions for " + finfo );
			return;
		}

		for ( i = 0, numWritten = 0; i < extArray.length; i++ ) {
			if ( extArray[ i ] != null ) {
				System.out.println( "at " + i + " is " + extArray[ i ] );
				++numWritten;
			}
		}
		
		if ( numWritten == 0 )
			System.out.println( "can't find extensions for " + finfo );
	}

/*
Try to tell a currently running process to open a file. After using a FileDialog to allow the
user to open a file, the full path name of that file is added as an argument to the open doc
command, and then the performCommand() method of the process is called to execute the command.
*/

	private static void tryOpenFile( AppProcess appProcess ) {
		AppCommand		theCommand;
		String			fileName;
		int				theErr;

		System.out.println( "" );
		System.out.println( "*** Telling process to open a file..." );

		if ( !appProcess.isRunning() ) {
			System.out.println( "process is not running" );
			return;
		}

		theCommand = appProcess.getCommand( AppCommand.kAppCommandOpenDoc );
		if ( theCommand == null ) {
			System.out.println( "can't get kAppCommandOpenDoc from appProcess" );
			return;
		}

		System.out.println( "theCommand = " + theCommand );

		fileName = JUtils.getFileNameFromUser();
		if ( fileName == null ) {
			System.out.println( "-- cancelled --" );
			return;
		}

		theCommand.addArg( fileName );

		System.out.println( "opening " + fileName );

		theErr = appProcess.performCommand( theCommand, 0 );

		System.out.println( "the error is " + theErr );
	}

/*
Try to find the applications whose name contains the given string.
*/

	private static void tryFindAppsByName( String name ) {
		AppFile			theApps[];
		int				i, numWritten;

		System.out.println( "" );
		System.out.println( "*** Finding apps whose name contains " + name );

		theApps = FileRegistry.getApps( name, 10 );
		if ( theApps == null ) {
			System.out.println( "can't find apps containing *" + name + "*" );
			return;
		}

		for ( i = 0, numWritten = 0; i < theApps.length; i++ ) {
			if ( theApps[ i ] != null ) {
				theApps[ i ].dumpInfo( System.out, "  " );
				++numWritten;
			}
		}
		
		if ( numWritten == 0 )
			System.out.println( "can't find apps containing *" + name + "*" );
	}

/*
Try to move the given process to the front.
*/

	private static void tryMoveAppToFront( AppProcess appProcess ) {
		int			theErr;
		
		System.out.println( "" );
		System.out.println( "*** Moving process to front..." );

		theErr = appProcess.move( null, AppProcess.kMoveToFront, 0 );
		
		if ( theErr != 0 )
			System.out.println( "theErr=" + theErr );
	}

/*
Try to display an application icon for the given file.
*/

	private static void tryDisplayApplicationIcon( AppFile appFile, int whichIcon ) {
		IconBundle			iconBundle;
		IconDisplayPanel	idp;
		FileType			fileType;
		int					bits[];
		IconDisplayFrame	aFrame;

		System.out.println( "" );
		System.out.println( "*** Displaying icon for application..." );

		iconBundle = appFile.getIconBundle();
		if ( iconBundle == null ) {
			System.out.println( "app has no IconBundle" );
			return;
		}

		fileType = new FileType( new FinderInfo( 0, JUtils.asciiToInt( "APPL" ) ) );

		bits = iconBundle.getIcon( fileType, whichIcon );
		if ( bits == null ) {
			System.out.println( "iconBundle.getIcon returned null" );
			return;
		}

		idp = new IconDisplayPanel( bits, iconBundle.getIconWidth( whichIcon ),
											iconBundle.getIconHeight( whichIcon ) );

		aFrame = new IconDisplayFrame( "Icon Test" );
		aFrame.setLayout( new BorderLayout( 0, 0 ) );
		aFrame.resize( 100, 100 );
		aFrame.add( "Center", idp );
		aFrame.show();
	}
}

class IconDisplayFrame extends Frame {
	public IconDisplayFrame( String ttl ) {
		super( ttl );
	}

	public boolean handleEvent( Event ev ) {
		if ( ev.id == Event.WINDOW_DESTROY ) {
			System.exit( 0 );
			return true;
		}
		else
			return false;
	}
}

/*
Used to display a panel containing an icon.
*/

class IconDisplayPanel extends Panel {
	MemoryImageSource	mis;
	int					bits[];
	int					width, height;

	public IconDisplayPanel( int bits[], int wd, int ht ) {
		this.bits = bits;
		width = wd;
		height = ht;
	}

	public Dimension preferredSize() {
		return new Dimension( 100, 100 );
	}

	public void paint( Graphics g ) {
		Image			image;

		mis = new MemoryImageSource( width, height, bits, 0, width );

		image = createImage( mis );
		
		g.setColor( Color.red );
		g.fillRect( 0, 0, 100, 100 );
		g.drawImage( image, 0, 0, null );
	}
}
		
/*
Prints every 20th entry in the mappings database.
*/

class BasicCEVisitor implements ConfigEntryVisitor {
	int				count;
	PrintStream		ps;

	BasicCEVisitor( PrintStream p ) {
		ps = p;
		count = 0;
	}

	public void visit( ConfigEntry fd ) {
		if ( ( count % 20 ) == 0 ) {
			ps.println( "at " + count + " is:" );
			fd.dumpInfo( ps, "  " );
		}

		++count;
	}
}

